local addonInfo, privateVars = ...

privateVars.lootScannerState = false

privateVars.internal.LootScanner = {}

function privateVars.internal.buildLootScannerPanel(frame, uiObject)
	
	local gridBG = UI.CreateFrame('Texture', frame:GetName() .. '.gridBG', frame)
	gridBG:SetLayer(1)
	gridBG:SetTextureAsync('nkAdvisor_Loot', 'gfx/databaseGridBG.png')
	gridBG:SetWidth(990)
	gridBG:SetHeight(400)
	gridBG:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, 35)
		
	local grid = EnKai.uiCreateFrame("nkGrid", frame:GetName() .. '.grid', frame)
			
	grid:SetPoint("TOPLEFT", gridBG, "TOPLEFT", 11, 0)
	grid:SetHeaderLabelColor(0.925, 0.894, 0.741, 1)
	grid:SetBorderColor(0, 0, 0, 1)
	grid:SetBodyColor(0.133, 0.133, 0.133, 1)
	grid:SetTransparentHeader()
	grid:SetLayer(3)
	grid:SetHeaderHeight(25)
	grid:SetHeaderFontSize(11)
	grid:SetFontSize(13)
	
	grid:SetSortable(true)
	
	local gridRows = 19
	
	local cols = 	{	{ align = 'center', texture = true, textureType = 'nkAdvisor_Loot', textureHeight = 14, textureWidth = 14, width = 20  }, 
						{ align = 'left', width = 300, header = "Item" },
						{ align = 'left', width = 200, header = "Zone" },
						{ align = 'left', width = 200, header = "NPC"	},
						{ align = 'center', texture = true, textureType = 'nkAdvisor_Loot', textureHeight = 14, textureWidth = 14, width = 20  }
					}	
	
	grid:SetVisible(false)	
	grid:Layout (cols, gridRows)	
								
	Command.Event.Attach(EnKai.events[frame:GetName() .. '.grid'].GridFinished, function ()
		privateVars.internal.LootScannerGridUpdate(frame, grid)
		grid:SetVisible(true)
	end, frame:GetName() .. '.grid.GridFinished')
	
	 Command.Event.Attach(EnKai.events[frame:GetName() .. '.grid'].WheelForward, function (_, rowPos)
		frame.slider:AdjustValue (rowPos)
	 end, frame:GetName() .. '.grid.Grid.WheelForward')
	
	 Command.Event.Attach(EnKai.events[frame:GetName() .. '.grid'].WheelBack, function (_, rowPos)
		frame.slider:AdjustValue (rowPos)
	 end, frame:GetName() .. '.grid.Grid.WheelBack')
	 
	 Command.Event.Attach(EnKai.events[frame:GetName() .. '.grid'].MouseOver, function (_, rowPos)
	 	local parent = grid:GetCell(rowPos, 1)
	 	local key = grid:GetKey(grid:GetRowPos() + rowPos -1)
	 	if key == nil then return end
	 	local itemData = nkItemBase.Query.byKey (key, false, nil, false)
	 	
	 	if itemData == nil then return end
	 	
	 	local mouse = Inspect.Mouse()
	 	local xpos, ypos = mouse.x - grid:GetLeft() + 20, mouse.y - grid:GetTop()
	 	
		nkAAPI.createTooltip(itemData.akey, itemData, grid, xpos, ypos, true)
	 end, frame:GetName() .. '.grid.Grid.MouseOver')
	 
	 Command.Event.Attach(EnKai.events[frame:GetName() .. '.grid'].MouseOut, function (_, rowPos)
		nkAAPI.hideTooltip()
	 end, frame:GetName() .. '.grid.Grid.MouseOut')
	 
	 Command.Event.Attach(EnKai.events[frame:GetName() .. '.grid'].LeftClick, function (_, row)
		local key = grid:GetKey(row)
		if key == nil then return end
		
		local xpos = Inspect.Mouse().x - grid:GetLeft()
		if xpos >= 720 and xpos <= 740 then 
			for idx = 1, #nkAM_NewLoot.loot, 1 do
				if nkAM_NewLoot.loot[idx].riftKey == key or nkAM_NewLoot.loot[idx].nkKey == key then
					table.remove(nkAM_NewLoot.loot, idx)
					privateVars.internal.LootScannerGridUpdate(frame, grid)
					break
				end
			end
		else			
			local cellValue = grid:GetCellValue(row, 1)
			
			local active = false
			
			if cellValue == 'gfx/statusRed.png' then
				grid:SetCellValue(row, 1, 'gfx/statusGreen.png')
				active = true
			else
				grid:SetCellValue(row, 1, 'gfx/statusRed.png')
				active = false
			end
			
			if grid:GetKey(row) == nil then
				EnKai.tools.error.display (addonInfo.toc.Identifier, string.format("Cannot get key for row #%d", row), 1)
				return
			end
			
			for idx = 1, #nkAM_NewLoot.loot, 1 do
				if nkAM_NewLoot.loot[idx].riftKey == key or nkAM_NewLoot.loot[idx].nkKey == key then
					nkAM_NewLoot.loot[idx].broadcast = active
					break
				end
			end
		end
	end, frame:GetName() .. '.grid.LeftClick')
	 
	frame.slider = EnKai.uiCreateFrame("nkScrollbox", frame:GetName() .. ".slider", frame)	
		
	frame.slider:SetPoint("TOPRIGHT", gridBG, "TOPRIGHT", -5, 25)
	frame.slider:SetHeight(grid:GetHeight() - 25)
	frame.slider:SetRange(1, 1)
	frame.slider:SetVisible(false)
	frame.slider:SetLayer(2)
	frame.slider:SetColor( nkAdvisor.layout.elementColor[1], nkAdvisor.layout.elementColor[2], nkAdvisor.layout.elementColor[3], nkAdvisor.layout.elementColor[4] )
	
	Command.Event.Attach(EnKai.events[frame:GetName() .. ".slider"].ScrollboxChanged, function ()		
		grid:SetRowPos(math.floor(frame.slider:GetValue('value')), true)
	end, frame:GetName() .. 'slider.ScrollboxChanged')
	
	local toggle = EnKai.uiCreateFrame("nkRadioButton", frame:GetName() .. '.toggle', frame)
	toggle:SetPoint("TOPLEFT", frame, "TOPLEFT", 13, 10)
	toggle:SetWidth(300)
	toggle:SetLabelWidth(100)
	toggle:SetText(privateVars.langTexts.lootScanner)
	toggle:SetSelection({{ value = true, label = privateVars.langTexts.active}, { value = false, label = privateVars.langTexts.inactive}})
	toggle:SetSelectedValue(false)
	toggle:SetColor ( nkAdvisor.layout.elementColor[1], nkAdvisor.layout.elementColor[2], nkAdvisor.layout.elementColor[3], nkAdvisor.layout.elementColor[4] )
	toggle:SetLabelColor(nkAdvisor.layout.labelColor[1], nkAdvisor.layout.labelColor[2], nkAdvisor.layout.labelColor[3], nkAdvisor.layout.labelColor[4] )
	
	Command.Event.Attach(EnKai.events[frame:GetName() .. '.toggle'].RadiobuttonChanged, function (_, newValue)
		privateVars.internal.init( newValue )
	end, frame:GetName() .. '.toggle.RadiobuttonChanged')
	
	local zone = UI.CreateFrame('Text', frame:GetName() .. '.zone', frame)
	zone:SetPoint("CENTERLEFT", toggle, "CENTERRIGHT", 10, 0)
	zone:SetFontSize(13)
	zone:SetFontColor(nkAdvisor.layout.labelColor[1], nkAdvisor.layout.labelColor[2], nkAdvisor.layout.labelColor[3], nkAdvisor.layout.labelColor[4] )
	
	if privateVars.currentZone ~= nil then
		zone:SetText(privateVars.currentZone.name)
	end
	
	local zoneButton = UI.CreateFrame('RiftButton', frame:GetName() .. '.zoneButton', frame)
	zoneButton:SetPoint("CENTERLEFT", toggle, "CENTERRIGHT", 200, 0)
	zoneButton:SetText(privateVars.langTexts.zoneButton)
	
	zoneButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
		privateVars.player = nil
		privateVars.currentZone = nil
		privateVars.internal.updateInfo ()
	end, frame:GetName() .. '.nkAAPI.Left.Click')
	
	-- not super nicely done but works
	
	privateVars.tempFrame = frame
	privateVars.tempGrid = grid
	privateVars.tempZone = zone
		
end


function privateVars.internal.LootScannerGridUpdate(frame, grid)

	local gridValues = {}
			
	for k, v in pairs (nkAM_NewLoot.loot) do
		local key = v.nkKey
		if key == nil then key = v.riftKey end

		local gfx = 'gfx/statusRed.png'
		if v.broadcast == true then gfx = 'gfx/statusGreen.png' end

		table.insert(gridValues, { { key = key, value = gfx }, { value = v.name}, { value = v.zone.name }, { value = v.npc }, { value = 'gfx/iconCancel.png' }})
	end
	
	grid:SetRowPos(1, false)
	grid:SetCellValues(gridValues)
		
	local maxCount = #gridValues
		
	if (maxCount-18) > 0 then
		frame.slider:SetRange (1, maxCount-18)
		frame.slider:SetVisible(true)
	else
		frame.slider:SetVisible(false)
	end

end

function privateVars.internal.init( flag )

	if flag == true then
		Command.Event.Attach(Event.Tooltip, privateVars.internal.LootScanner.tooltipHandler, "privateVars.internal.LootScanner.tooltipHandler")
		Command.Event.Attach(Event.Unit.Detail.Zone, privateVars.internal.LootScanner.zoneChangeHandler, "privateVars.internal.LootScanner.zoneChangeHandler")
		Command.Event.Attach(Event.Unit.Availability.Full, privateVars.internal.LootScanner.unitAvailableHandler, "privateVars.internal.LootScanner.unitAvailableHandler")
		
		privateVars.internal.updateInfo ()
		
	else
		Command.Event.Detach(Event.Tooltip, nil, "privateVars.internal.LootScanner.tooltipHandler")
		Command.Event.Detach(Event.Unit.Detail.Zone, nil, "privateVars.internal.LootScanner.zoneChangeHandler")
		Command.Event.Detach(Event.Unit.Availability.Full, nil, "privateVars.internal.LootScanner.unitAvailableHandler")
	end


end

function privateVars.internal.updateInfo ()

	if privateVars.player == nil then
		local details = Inspect.Unit.Detail('player')
		if details ~= nil and details.availability == "full" then
			privateVars.player = details
		end
	end
	
	if privateVars.player ~= nil then
		local zoneDetail = Inspect.Zone.Detail(privateVars.player.zone)
		
		privateVars.tempZone:ClearWidth()
		
		if zoneDetail ~= nil and zoneDetail.type == 'instance' then
			privateVars.currentZone = zoneDetail			
			privateVars.tempZone:SetText(privateVars.currentZone.name)
		else
			privateVars.currentZone = nil
			if zoneDetail == nil then
				privateVars.tempZone:SetText(privateVars.langTexts.ZoneUnknown)
			else
				privateVars.tempZone:SetText(privateVars.langTexts.zoneNoInstance)
			end			
		end
	end		

end

function privateVars.internal.LootScanner.unitAvailableHandler(_, units)
		
	if privateVars.player ~= nil then return end
	
	for k, v in pairs (units) do
		if v == 'player' then
			local details = Inspect.Unit.Detail('player')
			privateVars.player = details
			return
		end
	end
	
end

function privateVars.internal.LootScanner.tooltipHandler(_, tttype, shown, buff)

	if privateVars.currentZone == nil then return end
	if privateVars.player == nil then return end
	
	if tttype == 'item' or tttype == 'itemtype' then
		local itemDetails = Inspect.Item.Detail(shown)
		if itemDetails ~= nil and itemDetails.requiredLevel ~= nil and itemDetails.requiredLevel == 60 then
			local err, sourceSlot = pcall(Inspect.Item.Find, shown)
			
			if err == false or sourceSlot == nil then -- item not in inventory of player
		
				local itemData = nkItemBase.Query.byKey (itemDetails.type, true)
				if itemData == nil then
					-- item not known in db => completely new!
					privateVars.internal.LootScanner.newLoot(nil, itemDetails.id, itemDetails.name)
				else
					-- item known in db => check if loot table is known
					
					--if nkAAPI.isModuleRegistered ('nkAdvisor_Loot') == true then								
						-- local dropLine1, dropLine2 = nkAAPI.callModuleFunc('nkAdvisor_Loot', 'getDropLocation', {key = itemData.key})
						local dropLine1, dropLine2 = privateVars.internal.getDropLocation ({key = itemData.key})
						if dropLine1 == "" and dropLine2 ~= "" then
							-- unknown drop location
							privateVars.internal.LootScanner.newLoot(itemData.key, itemDetails.id, itemDetails.name)
						end
					--end
				end
			end
		else
			-- no item details, ignore for the moment
		end
	elseif tttype == 'unit' then
		local details = Inspect.Unit.Detail(shown)
		if details ~= nil and details.relation == 'hostile' then
			privateVars.lastTarget = details
		end
	end
		
end

function privateVars.internal.LootScanner.newLoot(itemDBKey, riftKey, name)

	if privateVars.lastTarget == nil then return end
	
	if nkAM_NewLoot.player[privateVars.player.name .. privateVars.player.guild] == nil then
		nkAM_NewLoot.player[privateVars.player.name .. privateVars.player.guild] = { name = privateVars.player.name, nameSecondary = privateVars.player.nameSecondary, guild = privateVars.player.guild }
	end
	
	local isNewLoot = true
	
	for idx = 1, #nkAM_NewLoot.loot, 1 do
		if nkAM_NewLoot.loot[idx].riftKey == riftKey then
			isNewLoot = false
			break
		end
	end
	
	if isNewLoot == false then return end
	
	local lootInfo = { nkKey = itemDBKey, riftKey = riftKey, broadcast = false, name = name, npc = privateVars.lastTarget.name, zone = privateVars.currentZone, player = privateVars.player.name .. privateVars.player.guild} 
	table.insert (nkAM_NewLoot.loot, lootInfo)
	
	privateVars.internal.LootScannerGridUpdate(privateVars.tempFrame, privateVars.tempGrid)

end

function privateVars.internal.LootScanner.zoneChangeHandler(_, units)

	if privateVars.player == nil then return end

	for k, v in pairs(units) do
		if k == privateVars.player.id then
			local zoneDetail = Inspect.Zone.Detail(v)
			if zoneDetail ~= nil and zoneDetail.type == 'instance' then
				privateVars.currentZone = zoneDetail
			else
				privateVars.currentZone = nil
			end
		end
	end

end